from PySZG import *
import random
import time

class navigator:
    def __init__(self):
	self.node = None
	self.lastTime = -1
	self.speed = 1.0
	self.threshold = 0.4

    def update(self, framework, time):
	if self.lastTime < 0:
	    self.lastTime = time
	    return
	
	wandDir = ar_ERM(framework.getMatrix(1))*arVector3(0,0,-1)
	wandTransDir = ar_ERM(framework.getMatrix(1))*arVector3(1,0,0)

	deltaX = framework.getAxis(0)
	deltaY = framework.getAxis(1)
	
	if abs(deltaX) < self.threshold:
	    deltaX = 0
	else:
	    deltaX = (deltaX - self.threshold) / (1.0 - self.threshold)
	if abs(deltaY) < self.threshold:
	    deltaY = 0
	else:
	    deltaY = (deltaY - self.threshold) / (1.0 - self.threshold)
	t = (wandDir * deltaY + wandTransDir * deltaX) * (time - self.lastTime) * self.speed 
	self.node.set(ar_TM(-t)*self.node.get())
	self.lastTime = time

#Used to read in input
class tool:
    def __init__(self):
        #These are set true when a button is pressed
        #Must be reset outside of this class
	self.pressLeft = False
        self.pressRight = False
        self.pressLeft2 = False
        self.pressRight2 = False

    #Called by eventprocess
    def update(self, framework, event):
        #Only respond to button events when it is a
        #Mousedown event
	if event.getType() == AR_EVENT_BUTTON and event.getButton() == 0:
	    #getIndex = which button
            #In input simulator, different button sets
            #are selected by hitting space bar while in
            #mode 4 or 5.
	    if event.getIndex() == 0:
		self.pressLeft = True
	    elif event.getIndex() == 1:
                self.pressRight = True
            elif event.getIndex() == 2:
		self.pressLeft2 = True
	    elif event.getIndex() == 3:
		self.pressRight2 = True

    #SZG Framework calls this with an eventqueue
    def eventProcessing(self, framework, eventQueue):
        while not eventQueue.empty():
            e = eventQueue.popNextEvent()
            self.update(framework, e)
        return 1

f = arDistSceneGraphFramework();

#Make the tool so we can read input
theTool = tool()

#Set the event queue callback
f.setEventQueueCallback(theTool.eventProcessing)

if len(sys.argv) < 2:
    print("buildworld must be called with the name of at least one other worldpart!")
    sys.exit()

if f.init(sys.argv) != 1:
    #Oh no....framework failed to initialize
    sys.exit()

#Must be operating a phleet
if f.getStandalone():
    print("This program only works in cluster mode. Please log in.")
    sys.exit()

#This is where the fun stuff begins
peer = arGraphicsPeer()
peer.setName("mainworld")
peer.queueData(0)
peer.init(f.getSZGClient())
peer.setTexturePath(f.getSZGClient().getAttribute("SZG_RENDER","texture_path"))
peer.loadAlphabet(f.getSZGClient().getAttribute("SZG_RENDER","text_path"))
#Set the peer to use our database
peer.setBridge(f.getDatabase())

worldRoot = f.getDatabase().getRoot()

nav = navigator()
nav.node = worldRoot.new("transform")

#Set the root node from which we share
peer.setBridgeRootMapNode(nav.node)

if peer.start() != 1:
    sys.exit()
if f.start() != 1:
    sys.exit()

for i in range(1, len(sys.argv)):
    if peer.connectToPeer(sys.argv[i]) < 0:
        print("Failed to connect to specified peer: " + sys.argv[i] + ".")
        sys.exit()
    else:
        print("Downloading from peer: " + sys.argv[i] + ".")
        if peer.pullSerial(sys.argv[i], 0, 0, AR_TRANSIENT_NODE, AR_TRANSIENT_NODE, AR_IGNORE_NODE) == 0:
            print("Error in downloading")

worldRoot = peer.getRoot()

l = nav.node.new("light")
l2 = nav.node.new("light")

light = arLight()
light.position = arVector4(-15,15,0,1)
light.diffuse = arVector3(1,1,1)
light.ambient = arVector3(0,0,0)
light.lightID = 0
l.set(light)

light = arLight()
light.position = arVector4(20,20,0,0)
light.ambient = arVector3(.1,.1,.1)
light.lightID = 1
l2.set(light)

'''
floormat = worldRoot.new("material")
mat = arMaterial();
mat.diffuse = arVector3(0,0,1)
floormat.set(mat)
floortrans = nav.node.new("transform")
floortrans.set(ar_SM(10, .1, 10))
texture = floortrans.new("texture")
texture.set("c:/szg-1.0/doc/python/texture/grass.jpg")
floor = arCubeMesh()
floor.attachMesh(texture)

trans = worldRoot.new("transform")
trans.set(ar_TM(2, 4, -3))

mat = trans.new("material")
material = arMaterial()
material.diffuse = arVector3(1,0,0)
mat.set(material)
part = arSphereMesh(50)
part.attachMesh(mat)

'''
while 1:
    f.setViewer()
    f.setPlayer()
    f.processEventQueue()

    nav.update(f, time.clock())
        
    f.loadNavMatrix()
    time.sleep(0.05)
